/*-- Reads in waveform data records (wdr) in the "c20" format specified by the
	structure in cryosat20hz.h then computes 1-second statistics --*/ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "geosat10hz.h"

int main( int   argc, char *argv[] )
{

	double mean(int, double[]);
	double median(int, double[]);
	double stdev(int, double[]);

	int		nwav = 10; /* use 22 for CryoSat and 20 for Envisat */
	int		tide, delay, hgt, h0, issb, ioff, nstr, tide_csr;
	int		loop, wavno, ns=0;
	
	double	lat[nwav], lon[nwav];
	double	hmdiff[nwav], hmdiffreg[nwav], swh20[nwav];
	double	slat, swh, slon, tnow, tprev, hstdev, hsreg, swh2;
	
	struct GEOSAT10HZ c;
	FILE	*fpin;

	if (argc != 3 || (fpin = fopen(argv[1], "r")) == NULL) {
		fprintf (stderr, "usage:  c20_1stat_geo  input.g10 nstruct  \n");
        	fprintf (stderr, "        nstruct (1)-use Geosat on-board range \n");
        	fprintf (stderr, "        nstruct (2)-use 2-parameter retracking result \n");
        	fprintf (stderr, "        nstruct (3)-use 3-parameter retracking result \n");
		return EXIT_FAILURE;
	}
	nstr = atoi(argv[2]);
	tprev = 0.0;
	
	while( loop == 1 ){
		loop = 0;
		
		for(wavno=0; wavno<nwav; wavno++){
			if((fread ((void *)&c, sizeof(struct GEOSAT10HZ), (size_t)1, fpin)) != 1){
				/* fprintf(stderr, "Error reading record from file \n"); */
				goto endpoint; 
			}
			tnow = 1.0e-09*(double)c.nanos+(double)c.tai85s;
			
			/* detect breaks in time between passes */
			if ((ns != 0) & ((tnow-tprev) > 4.0)) {
				goto endloop;
			}
					
			/* editing criteria for retracking errors */
			if(c.trackbits != 0 ) {
				goto endloop;
	  		}
	  		
			lat[wavno] = 1.0e-07 * (double)c.lat;
			lon[wavno] = 1.0e-07 * (double)c.lon;
			if(lon[wavno] < 0) lon[wavno]=lon[wavno]+360.;

                        tide = c.hltide + c.hstide + c.hptide + c.hotide;
			h0 =c.alt-(c.range+tide); 
			
			hgt = 0.0;
			
	  		if(nstr == 1) {
   				hgt = (double)h0; 
   			}
   			
   			if(nstr == 2) {
   				hgt = (double)h0 - (double)c.drange[1];
			}
			
			if(nstr == 3) {
				hgt = (double)h0 - (double)c.drange[0];
			}
        	
        		hmdiff[wavno] = (hgt-(double)c.mss);
                       	swh20[wavno] = c.pswh[0];
        		tprev=tnow;
		}
		
		slat = median(nwav, lat);
		slon = median(nwav, lon);
                swh = median(nwav, swh20);

/* factor to account for the multiple waveform retracking.
   on average there are 2.0 points used so we should multiply all numbers by 1.41 */

		hstdev = 1.4*stdev(nwav, hmdiff);
		

		fprintf(stdout,"%lf %f %f %f %f \n", tnow, slon, slat, hstdev, swh);
		
		endloop:
		tprev=tnow;
		loop = 1;
		ns++;
	}
	
	endpoint:
	fclose (fpin);
	fprintf(stderr, "Number of 1 Hz records: %d \n", ns);
	return EXIT_SUCCESS;
}

double mean(int nw, double htd[]) {
    double	sum=0;
    int		i;
    
    for(i=0; i<nw; i++)
        sum = sum + htd[i];
    return((double)sum/nw);
}
 
double median(int nw, double sld[]) {
    double 	temp;
    int 	i, j;
    // the following two loops sort the array x in ascending order
    for(i=0; i<nw-1; i++) {
        for(j=i+1; j<nw; j++) {
            if(sld[j] < sld[i]) {
                // swap elements
                temp = sld[i];
                sld[i] = sld[j];
                sld[j] = temp;
            }
        }
    }
 
    if(nw%2==0) {
        // if there is an even number of elements, return mean of the two elements in the middle
        return((sld[nw/2] + sld[nw/2 - 1]) / 2.0);
    } else {
        // else return the element in the middle
        return sld[nw/2];
    }
}

double stdev(int nw, double htd[]) {
    double	sum=0.0, mht=0.0;
    int		i;
    
    mht = mean(nw, htd); 
    
    for(i=0; i<nw; i++)
        sum = sum + ((htd[i]-mht)*(htd[i]-mht));
    return(sqrt((double)sum/(nw-1)));

}

/* double *linreg(int nw, double yi[], double xd[], double ye[]) {
	double my=0.0, mx=0.0;
	double slope, sloped=0.0, slopen=0.0, inter;
	int i;

	my = mean(nw,yi);
	mx = mean(nw,xd);
	
	for(i=0; i<nw; i++) {
		sloped = sloped + ((xd[i]-mx)*(yi[i]-my));
		slopen = slopen + ((xd[i]-mx)*(xd[i]-mx));
	}
	
	slope = sloped/slopen;
	
	inter = my - (slope*mx);
	
	for(i=0; i<nw; i++) {
		ye[i] = inter + (slope*xd[i]);
	}
	
	return ye;
} */
