/*  c20_dump.c input.c20 > output

program to read a Cryosat 20 Hz
and time, lon, lat, height, and mean_ssh

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cryosat20hz.h"

int main (int argc, char **argv) {

	struct CRYOSAT20HZ c;
	FILE	*fpin;
	int	i, n, k, tide, delay, h, h0, h1, h2,issb, kk, ioff, nstr, tide_csr;
        int     ngood=0, nbad=0;
	int	range;
	double	t,lat, lon, t0, alt, diff, wfac, sfac=0., swdiff, damp;
        double	swh, mss, ssha, ssh, ranged, altd, tide_out;
	double  rhtd, rhd1, rmean, rm2, rms;
        double  rht[20];
	
	/* Make sure usage is right by opening files  */
	
	if (argc != 3 || (fpin = fopen(argv[1], "r")) == NULL) {
		fprintf (stderr, "usage:  c20_pro input.c20 nstruct  \n");
                fprintf (stderr, "        nstruct (1)-use CryoSat on-board range \n");
                fprintf (stderr, "        nstruct (2)-use 2-parameter retracking result \n");
                fprintf (stderr, "        nstruct (3)-use 3-parameter retracking result \n");
                fprintf (stderr, "        nstruct (4)-use threshold retracking result \n");
		exit (-1);
	}
        nstr = atoi(argv[2]);
	k = 0;
	n = 0;
		
	while ( (fread ((void *)&c, sizeof(struct CRYOSAT20HZ), (size_t)1, fpin)) == 1 ) {
		if(k == 0) t0 = 1.0e-06 * c.microsec + c.sec2000;
        k=k+1;
	t = 1.0e-06 * c.microsec + (double)c.sec2000;
	lat = 1.0e-06 * c.lat;
	lon = 1.0e-06 * c.lon;
	tide = c.hltide + c.hstide + c.hptide + c.hotide;
        tide_csr = c.new_tide + c.hstide;
    
/*	delay = c.hwet + c.hdry + c.hiono + c.hinvb; don't use ionosphere for envisat */
	delay = c.hwet + c.hdry + c.hiono; // corrected range based on Jason-1 GDR
        ioff = -3440.0; /* based on analysis by Remko Scharroo at 2011 Crypsat meeting */
        issb = floor(0.060*c.pswh[0]+.5);
	h0 =c.alt-(c.range+c.hdopp+tide_csr+delay+ioff)+issb;
	range = c.range + delay;
        swdiff = abs(c.pswh[1]-c.pswh[0]);

   	if(nstr == 1){
		h = h0;
		range = c.range;
        }
   	if(nstr == 2){
		h = h0 - c.drange[1];
		range = c.range + c.drange[1];
        }    
	if(nstr == 3){
		h = h0 - c.drange[0];
		range = c.range + c.drange[0];
        }        
        if(nstr == 4){
		h = h0 - c.drange[2];
		range = c.range + c.drange[2];
        }

//	convert to meters for acii output
	ranged = 1.0e-3*(double)range;
	altd = 1.0e-3*(double)c.alt;
	ssh = 1.0e-3*(double)h;
	tide_out = 1.0e-3*(double)tide_csr;
        
	if(c.trackbits == 0) {
		printf("%lf %lf %lf %lf %lf %lf \n",lat,lon,ranged,altd,ssh,tide_out);
        	ngood = ngood +1;
        } 
        else {
        	nbad=nbad+1;
        }

	}
	fclose (fpin);
	fprintf(stderr," ngood nbad %d %d \n",ngood,nbad);
	exit (0);
}
