!
      program c20_smooth
!
!  program to read cryosat altimeter data in c20 format.
!  divide the data into passes when a time gap exceeds 3 sec
!  low-pass filter the swh and amp
!  write out the data in c20 format
!
      parameter (ND=5000000,NF=200, NFA=60)
      implicit real*8 (d)

include "ca_structs.inc"

      type(reaa) c20
      type(reaa) c20b(ND)
      real*8  dm(ND)
      real*4  wf(NF),wfa(NFA)
      integer*4 iswh(ND),iamp(ND),inoise(ND),flag(ND)
      character*80      infile /' '/,   outfile /' '/
!
      mdata=0
      ip=0
!
      narg = iargc()
      if(narg.lt.2) then
        write(*,5)
 5      format ('c20_smooth infile.c20 outfile.c20'/)
        stop
      else
        call getarg(1,infile)
        call getarg(2,outfile)
      endif
!
        nc=index(infile,' ')
        infile(nc:nc)=char(0)
        nc=index(outfile,' ')
        outfile(nc:nc)=char(0)
        call getfil(4,lui,infile,istat)
        if(istat.lt.0) go to 999
        call getfil(3,luo,outfile,istat)
        if(istat.lt.0) go to 999
!
!   make the Gaussian filter for swh 0.5 gain at 45 km
!
      nf2=NF/2
      nsig=NF/8
      fac=0.5/(nsig*nsig)
      do 10 i=1,NF
      id=i-nf2
      wf(i)=exp(-(id*id)*fac)
  10  continue
!
!   make the Gaussian filter for noise and amp 0.5 gain at 13.6 km (2.0 sec)
!
      nfa2=NFA/2
      nsiga=NFA/8
      fac=0.5/(nsiga*nsiga)
      do 11 i=1,NFA
      id=i-nfa2
      wfa(i)=exp(-(id*id)*fac)
  11  continue
!
!  read a pass
!
  15  continue
      if(mdata.gt.0) ip=1
!
  20  continue
!
!  read a 360-byte record.
!
      ip=ip+1
      call rddisc(lui,c20b(ip),100,istin)
      if(istin.lt.0) go to 50
      mdata=mdata+1
!
!  decode a time record and store the swh unless the 
!  data are flagged
!
      c20=c20b(ip)
      dm(ip)=c20%sec2000+c20%microsec*1.d-06
      iswh(ip)=c20%pswh(1)
      iamp(ip)=c20%pamp(1)
      inoise(ip)=0
      dlat=c20%lat*1.d-06
      dlon=c20%lon*1.d-06
      flag(ip)=c20%trackbits
!
      if(ip.eq.1) go to 20
!
!  check for too many points
!
      if(ip.gt.ND) then
      write(*,905)
 905  format(' error ! # of points exceeded ')
      stop
      endif
!
!  check for significant time gap
!
      ddtm=dm(ip)-dm(ip-1)
      if(ddtm.gt.3.d0.or.ddtm.le.0.d0) go to 50
      go to 20
!
!  filter and write the pass
!
   50 continue
!
!  apply a low-pass Gaussian filter to the wave height
!  this filter is 200 points long  The wavelength
!  where the filter reaches 0.5 gain is 90 km.
!
      ipm1=ip-1
      do 54 i=1,ipm1
!
!   filter swh
!
      sum=0.
      sumw=0.
      k0=max(1,i-nf2)
      kf=min(ipm1,i+nf2-1)
      do 51 k=k0,kf
      iw=k-i+nf2+1
      if(flag(k).gt.0) go to 51
      sum=sum+iswh(k)*wf(iw)
      sumw=sumw+wf(iw)
   51 continue
      c20=c20b(i)
      if(sumw.gt.0.) then
      c20%pswh(2)=nint(sum/sumw)
      endif
!
!   filter noise and amp
!
      sumn=0.
      sum=0.
      sumw=0.
      k0=max(1,i-nfa2)
      kf=min(ipm1,i+nfa2-1)
      do 53 k=k0,kf
      iw=k-i+nfa2+1
      if(flag(k).gt.0) go to 53
      sum=sum+iamp(k)*wfa(iw)
      sumn=sumn+inoise(k)*wfa(iw)
      sumw=sumw+wfa(iw)
   53 continue
      if(sumw.gt.0.) then
      c20%pamp(2)=nint(sum/sumw)
      c20%pnoise(2)=nint(sumn/sumw)
      else
      c20%pamp(2)=65000
      c20%pnoise(2)=0
      endif
      c20%trackbits=flag(i)
!
      call wrdisc(luo,c20,100)
   54 continue
!
!  stuff the last record read into the first slot.
!
      if(istin.lt.0) go to 999
      dm(1)=dm(ip)
      iswh(1)=iswh(ip)
      iamp(1)=iamp(ip)
      inoise(1)=inoise(ip)
      flag(1)=flag(ip)
      c20b(1)=c20b(ip)
      go to 15
!
!  close the files.
!
 999  continue
      call frefil(2,lui,istat)
      call frefil(2,luo,istat)
      close(luo)
      close(lui)
      write(*,911)mdata
 911  format(' # of data points processed, c20_smooth ',i12)
      stop
      end
!
