/*-- Reads in waveform data records (wdr) in the "c20" format specified by the
	structure in cryosat20hz.h then calls retracking routines --*/ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cryosat20hz.h"

int main( int   argc, char *argv[] )
{
	/*-- set odd number of waveforms --*/
	int nw = 3;
	int nwi = nw-1;
	int bufpt = 128*nw;
	int dpt, gate, indx, istat, loop, ni, niter, no, npar, mode, nw2, wavno, readrec;
	double rw[bufpt], rmw[bufpt];
	double	t0[nw],tswap[nw], t1[nw];
	int iwf[nw];
	double drange, sig0, sigc2, sigw, sig1, amp, std, swh, prob, alph = 0.0090;
	double fac = 3.3e6, gt2mm = 468.257;
	int ming = 9, maxg = 109;
	struct CRYOSAT20HZ rea;
	struct CRYOSAT20HZ rbuf[nw];
	int	ngood=0, nbad=0;
	unsigned int flag, itest, bitval;
	FILE *fpin, *fpout;
	

	if( argc != 4 )
    {
        printf( "usage: %s infile.c20 outfile.c20 npar \n", argv[0] );

        return EXIT_FAILURE;
    }
    /* Thus:
    	argv[1] - input filename
    	argv[2] - output filename
    	argv[3] - npar
    */
    
    npar = atoi(argv[3]);
    
	if(npar != 2 & npar != 3){
		fprintf(stderr,"npar not 2 or 3 \n");
		goto endpoint;
	}
	
	
	/*-- set index of 2-element arrays in CRYOSAT20HZ structure
		depending on whether a 2- or 3-parameter retracking is being done --*/
	indx=0;
	if(npar == 2){
		indx=1;
	}
	
	if((fpin = fopen(argv[1], "r")) == NULL){
		fprintf(stderr,"input file not opened \n");
		goto endpoint;	
	}
	
	if((fpout = fopen(argv[2], "w")) == NULL){
		fprintf(stderr,"output file not opened \n");
		goto endpoint;	
	}
	
	ni = 0;
	no = 0;
	nw2 = nwi/2;
	sigw = 0.5;
	
	for(wavno=0; wavno<nwi; wavno++){
		if((fread ((void *)&rea, sizeof(struct CRYOSAT20HZ), (size_t)1, fpin)) != 1){
			goto endpoint; 
		}
		rbuf[wavno] = rea;
		flag=rbuf[wavno].trackbits;
		iwf[wavno]=0;
		if(flag != 0){
			iwf[wavno]=1;
		} 
		if(npar == 2 & rea.pswh[1] != 32767){
			if(rea.drange[0] != 0) {
				t0[wavno] = 65. + ((double)rea.drange[0]/gt2mm);
			}
			else {
				t0[wavno] = 33.38;
			}
		}
		else {
			if(rea.drange[2] != 0) {
				t0[wavno] = 65. + ((double)rea.drange[2]/gt2mm);
			}
			else {
				t0[wavno] = 33.38;
			}
		}
		
		ni++;
	}
	
	
	readrec = 0;
	
	while(readrec == 0){
		if((fread ((void *)&rea, sizeof(struct CRYOSAT20HZ), (size_t)1, fpin)) != 1){
			readrec = 1;
			goto endpoint;
		}
		rbuf[nwi] = rea;
		flag=rbuf[nwi].trackbits;
		iwf[nwi]=0;
		if(flag != 0){
			iwf[nwi]=1;
		}
		if(npar == 2 & rea.pswh[1] != 32767){
			if(rea.drange[0] != 0) {
				t0[nwi] = 65. + ((double)rea.drange[0]/gt2mm);
			}
			else { 
				t0[nwi] = 33.38;
			}
		}
		else {
			if(rea.drange[2] != 0) {
				t0[nwi] = 65. + ((double)rea.drange[2]/gt2mm);
			}
			else {
				t0[nwi] = 33.38;
			}
		}
		
		
		ni++;
		
		/*-- Gather the surrounding waveforms because the data are
			stored as unsigned integer*2 */
		
		for(wavno=0; wavno<nw; wavno++){
			rea=rbuf[wavno];
			for(gate=0; gate<128; gate++){
				dpt=(128*wavno)+gate;
                 /* use the csum and fac to renormalize the waveforms */
                                if(rea.csum > 1000){
				  rw[dpt] = rea.wave[gate]*fac/rea.csum;
                                }
                                else {
				  rw[dpt] = rea.wave[gate]*fac/rea.csum;
                                }
			}
		}
		
		
		/*-- center the analysis on the record in the middle of nw waveforms --*/
		
		rea=rbuf[nw2];
		
		/*-- set the starting parameters for the waveform model --*/
		
		/*-- if npar is 2 then used smoothed swh from previous retrack --*/
		if(npar == 2 & rea.pswh[1] != 32767){
			sigc2 = ((rea.pswh[1])/4.0)*((rea.pswh[1])/4.0);
			sig0 = sqrt(sigc2+(263.0*263.0))/gt2mm;
		}
		/*-- if npar is 3, this retracker works best with a starting swh of 3000 mm --*/
		else {
			swh = 3000.0;
			sig0 = (swh/4.0)/gt2mm;
		}
		
		niter = 1;
		loop = 1;
				
		for(wavno=0; wavno<nw; wavno++){
			tswap[wavno]=t0[wavno];
		}

		while( loop == 1 ){
			loop = 0;
			if(niter > 1){
				for(wavno=0; wavno<nw; wavno++){
					t0[wavno] = t1[wavno];
				}
				sig0 = sig1;
			}
			/*-- call function for fitting model waveform --*/ 
			erfpick_(&nw,&sigw,rw,iwf,&t0,&sig0,&npar,&ming,&maxg,&alph,rmw,&t1,&sig1,&amp,&std,&prob);

			if(prob < 0.30 | niter < 5){
				
				/*-- bad if more than 10 iterations --*/
				if(niter >= 10){
					itest=rea.trackbits;
					bitval = 8192;
					if( (itest & 8192) == 0){
						rea.trackbits = rea.trackbits + 8192;
					}
					rea.pamp[indx] = 262144;
					rea.pswh[indx] = 32767;
					goto writetostruct;
				}
				niter = niter+1;
				loop = 1;
			}
		}
				
		/*-- don't change the rea if this is hopeless --*/
		
		if(std < -8.9){
				itest=rea.trackbits;
				if( (itest & 8192) == 0){
					rea.trackbits = rea.trackbits + 8192;
				}
				rea.pamp[indx] = 262144;
				rea.pswh[indx] = 32767;
				goto writetostruct;			
		}
		
		/*-- set value for chi-squared misfit measure --*/
		if(37267 < (int)(std*1000)){
			rea.rchisq[indx] = 37267;
		}
		else {
			rea.rchisq[indx] = (int)(std*1000);
		}

		/*-- flag the data if the std is out of range --*/
                if(std > 0.8){
                        itest=rea.trackbits;
                        if( (itest & 1024) == 0){
                                rea.trackbits = rea.trackbits + 1024;
                        }
                }

		
		/*-- flag the data if the amp is out of range --*/
		if(amp < 40000 | amp > 80000){
			itest=rea.trackbits;
			if( (itest & 16384) == 0){
				rea.trackbits = rea.trackbits + 16384;
				rea.pamp[indx] = 262144;
			}
		}
		else {
			rea.pamp[indx] = (int)amp;
		}

		
		/*-- update the drange, swh, and amp
			for Cryosat the two-way travel time is measured to the center of the
			window which corresponds to gate 65.0a. (e-mail from Cryosat Team 
			says gate 64 when counting 0-127).
		--*/
		
		drange=(t1[nw2]-65.0)*gt2mm;
		
		/*-- correct the wave height for the pulse length --*/
		sigc2 = ((sig1*gt2mm)*(sig1*gt2mm)) - (263*263);
		
		if(sigc2 <= 0.0) {
			sigc2 = 0.0;
		}
		swh = 4.0*sqrt(sigc2);
		
		/*-- check for drange out of bounds, don't update drange or swh --*/
		
		if(fabs(drange) >= 32767.0){
			rea.drange[indx] = 0;
			itest = rea.trackbits;
			if( (itest & 2048) == 0) {
				rea.trackbits = rea.trackbits + 2048;
			}
		}
		else {
			rea.drange[indx] = (int)drange;
		}
		
		/*-- don't trust small or large wave heights --*/
		
		if(swh <= 300.0 | swh > 10000.0){
			rea.pswh[indx] = 32767;
			itest = rea.trackbits;
			if( (itest & 4096) == 0) {
				rea.trackbits = rea.trackbits + 4096; 
			}
		}	
		else {
			rea.pswh[indx] = (int)swh;
		}
		
		
		/*-- set maximum amplitude --*/
		if(amp > 262144){
			rea.pamp[indx] = 262144;
		}
		else {
			rea.pamp[indx] = (int)amp;
		}

/* -- label --*/	
		writetostruct:
		swh = rea.pswh[indx];
		drange = rea.drange[indx];
		flag = rea.trackbits;
		amp = rea.pamp[indx];
		
		if ( (fwrite ((void *)&rea, sizeof(struct CRYOSAT20HZ), (size_t)1, fpout)) != 1) {
		  fprintf (stderr, "Error writing data to file. \n");
		  goto endpoint;
	  	}
	  		  	
	  	if(rea.trackbits == 0){ 
	  		ngood = ngood+1;
	  	}
	  	else {
	  		nbad = nbad+1;
	  	}	  	
		
		/*-- count number of output points --*/
		no = no+1;

		/*-- shift waveforms in buffer --*/
		for(wavno=0; wavno<nwi; wavno++){
			rbuf[wavno]=rbuf[wavno+1];
			iwf[wavno]=iwf[wavno+1];
			t0[wavno]=tswap[wavno+1];
		}

	}
	
/* -- label --*/
	endpoint:
	fprintf(stdout,"# of points: input - %d, output - %d \n",ni,no);	

	fclose(fpin);
	fclose(fpout);
	
	return EXIT_SUCCESS;
}
