/*  c20_mss_tide.c  input.c20  output.c20

program to read a Cryosat 20 Hz file,
add CSR 4.0 Ocean and Load tide, and GSFC00.1
mean sea surface, and write output.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cryosat20hz.h"

void main (int argc, char **argv) {

	struct CRYOSIN20HZ g;
	FILE	*fpin, *fpout;
	double	lat, lon, time2000, sh, tide;
	int	tot_tide;
	
	/* check usage */
	
  	if (argc != 3 || (fpin = fopen(argv[1], "r")) == NULL || (fpout = fopen(argv[2], "w")) == NULL) {
		fprintf (stderr, "usage:  c20_mss_tide  input.c20  output.c20\n");
		exit (-1);
	}
	
	/* Keep going while there are records in the input file:  */
	
	while ( (fread ((void *)&g, sizeof(struct CRYOSIN20HZ), (size_t)1, fpin)) == 1) {
	
		lat = 1.0e-06 * g.lat;
		lon = 1.0e-06 * g.lon;
		time2000 = g.sec2000 + 1.0e-06 * g.microsec;
		
		/* add the sea surface height */
		ssh_(&lat,&lon,&sh);
                g.mss=(int)rint(1000.*sh);

		/* add the new tide model */
		if( (csr40tide_(&time2000, &lat, &lon, &tide) == 0) ) {
			g.new_tide = (int)rint(tide);
		}
		else {
			g.new_tide = 32767;
		}
                tot_tide = g.hltide + g.hstide + g.hptide + g.hotide;
                /*printf(" %d %d \n",tot_tide,g.new_tide);*/

                /* add the wet/dry flag */
                if(iwetdry_(&lat, &lon) == 2 ) g.trackbits = g.trackbits + 2;

		if ( (fwrite ((void *)&g, sizeof(struct CRYOSIN20HZ), (size_t)1, fpout)) != 1) {
			fprintf (stderr, "c20_mss_tide:  Error writing data to %s\n", argv[2]);
			exit (-1);
		}
	}
	fclose (fpin);
	fclose (fpout);
	exit (0);
}
