/*  c20_thresh.c input.c20 > output  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cryosat20hz.h"

int main (int argc, char **argv) {

	struct CRYOSIN20HZ c;
	FILE	*fpin, *fpout;
	int		nw, j, nstr;
	int		swav[512];
	double	cgpow, norm, tlev, tlpow, tlgate, cgprev, dpow, dpowmin = 1/1.28e5;
	double  gt2mm=234.129; /* 234.129 468.257 */
	
	/* Make sure usage is right by opening files  */
	
	if (argc != 4 || (fpin = fopen(argv[1], "r")) == NULL || (fpout = fopen(argv[2], "w")) == NULL) {
		fprintf (stderr, "usage:  s20_thresh input.c20 output.c20 nstruct  \n");
                fprintf (stderr, "        nstruct (1)- LRM \n");
                fprintf (stderr, "        nstruct (2)- SAR \n");
		fprintf (stderr, "        nstruct (3)- SIN \n");
		return EXIT_FAILURE;
	}
	
	nstr = atoi(argv[3]);
	
	if(nstr == 1){
		tlev = 0.025;
	}
	
	if(nstr == 2){
		tlev = 0.125;
	}

	if(nstr == 3) {
		tlev = 0.125;
	}
	
    /*-- Initialize array containing cumulative sum of waveforms --*/
    for (j=0; j<512; j++){
	    	swav[j]=0.0;
    }
    
	nw = 0;	
		
	while ( (fread ((void *)&c, sizeof(struct CRYOSIN20HZ), (size_t)1, fpin)) == 1 ) {
		nw++;
		
	/*-- First loop to get cumulative sum --*/
        for (j=1; j<512; j++){
	    	swav[j]=swav[j-1]+c.wave[j];
        }
        
        /*-- 	Second loop to find threshold level
        		This works by finding minimum difference between element and tlpow --*/
        
        norm = (double)swav[511];
        tlgate = 0.0;

        /*-- determine when the cpower exceeds the threshold and interpolate --*/

        j = 0;
        cgpow = (double)swav[j]/norm;
        while (cgpow < tlev) {
          j = j + 1;
          if(j > 511) fprintf(stderr,"c20_thresh index out of bounds %d \n",j);
          cgpow = (double)swav[j]/norm;
        }
        cgprev = (double)swav[j-1]/norm;
        dpow=cgpow-cgprev;
        if(dpow > dpowmin){
          tlgate = (double)(j-1) + ((tlev-cgprev)/dpow);
        }
        else {
          tlgate = (double)(j-1) + 0.5;
        }
           
        c.drange[2] = (int)(((double)tlgate-65.0)*gt2mm);
        
    	if ( (fwrite ((void *)&c, sizeof(struct CRYOSIN20HZ), (size_t)1, fpout)) != 1) {
			fprintf (stderr, "Error writing data to %s\n", argv[2]);
			return EXIT_FAILURE;
		} 	
	}
	
	fclose(fpin);
	fclose(fpout);
	fprintf(stderr,"number of waveforms in file: %d \n",nw);
	return EXIT_SUCCESS;
}
