/*  s20_thresh.c input.s20 > output  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cryosat20hz.h"

int main (int argc, char **argv) {

	struct CRYOSIN20HZ c;
	FILE	*fpin, *fpout;
	
	double mean(int, int, double[]);
	double stdev(int, int, double[]);
	
	int		nw, i, j, k, bline, ngood, nbad, mwel, swel;
	int		nn=10; /* number of gates taken to estimate thermal noise */
	int		pnw[nn];
	double	thn; /* thermal noise level */
	int		nstart=14, nend=497; /* for clipping waveform at both ends to avoid Fourier wrap around issues */
	int		swstart, swend, swtest=1, filt;
	int		coridx, maxel;
	double	anum, aden, aval, maxamp, wp2; /* numerator of A, denominator of A, value of A, waveform power squared */
	double	q; /* threshold factor, Helm (2007) says 0.5 */
	double	cgpow, norm, tlev, tlpow, tlgate, cgprev, dpow, dpowmin = 1/1.28e5;
	double  gt2mm;
	double	wav[512], df1[512], df2[512];
	double	agccor, wpow, scpow, rpow, powsum, pp, ep1, ep2;
	unsigned int itest;
	
	const int agccortab[64] = { \
        6148, 6062, 5959, 5866, 5774, 5675, 5560, 5468, \
        5376, 5277, 5186, 5075, 4967, 4868, 4777, 4663, \
        4571, 4472, 4381, 4302, 4210, 4111, 3998, 3907, \
        3815, 3716, 3625, 3514, 3395, 3303, 3188, 3097, \
        3005, 2906, 2815, 2703, 2612, 2509, 2420, 2296, \
        2204, 2101, 2011, 1939, 1848, 1743, 1656, 1542, \
        1451, 1346, 1258, 1144, 1062, 941, 833, 742, \
        652, 544, 456, 342, 247, 147, 59, 0};
	
	/* Make sure usage is right by opening files  */
		
	if (argc != 5 || (fpin = fopen(argv[1], "r")) == NULL || (fpout = fopen(argv[2], "w")) == NULL) {
		fprintf (stderr, "usage:  %s input.c20 output.c20 baseline threshold  \n",argv[0]);
        fprintf (stderr, "        baseline (1)- Baseline A \n");
    	fprintf (stderr, "        baseline (2)- Baseline B \n");
		fprintf (stderr, "	  \n");
		fprintf (stderr, "	  threshold value (0.0-1.0) \n");
		return EXIT_FAILURE;
	}
	
	bline = atoi(argv[3]);
	q = atof(argv[4]);
	
	/* The following could use error handling: */
	if(bline != 1 && bline != 2) {
    	fprintf(stderr,"invalid baseline value \n");
        return EXIT_FAILURE;
    }
        
    switch (bline) {
    	case 1:
			gt2mm=468.257;    	
    		break;
    	case 2:
    		gt2mm=234.129;
    		break;
    }    
			
	nw = 0;	
	ngood = 0;
	nbad = 0;
	
	while ( (fread ((void *)&c, sizeof(struct CRYOSIN20HZ), (size_t)1, fpin)) == 1 ) {		
		nw++;
		
		/* FILTERING SCHEME: Discard waveform if AGC < 0 */
		if (c.pamp[0] < 0) {
            c.trackbits = 8192; 
		}	     
		
		
		if (c.pamp[0] == 0) 
			coridx = 63;
		else if (bline == 1) /* Baseline A */
			coridx = (int)((c.pamp[0] + c.pamp[1])/100);
		else
			coridx = 62 - (int)((c.pamp[0] - c.pamp[1])/100);
			 
			 
		agccor = 1.0e-2 * (double)agccortab[coridx];
	
	maxel = 0;
	maxamp = 0.0;
	powsum = 0.0;
	
	for (i=nstart; i<nend; i++) {
	    scpow = (double)c.wave[i]*c.esf_A*pow(2.0,c.esf_B);
        rpow = (10*log10(scpow)) + (0.01*(double)c.pamp[0]) - agccor;
		wav[i]=pow(10,0.1*rpow);
		
		df1[i]=wav[i+1]-wav[i];
		df2[i]=wav[i+2]-wav[i];
		powsum += wav[i];
		
		/* find maximum amplitude */
		if (wav[i] > maxamp) {
			maxel = i;
	 		maxamp = wav[i];
	 	} 
	}
	
	
	/* Compute pulse peakiness */
	pp = 15.0*maxamp/powsum;
	
	/* FILTERING SCHEME: Discard waveform if PP > 4 (might need tweaking) */
	if (pp > 4.0) {
    	c.trackbits = 8192; 
	} 
	
	/* compute epsilon parameters (Fenoglio-Marc, 2009), might need tweaking */
	ep1 = 0.2*stdev(nstart, nend, df2);
	ep2 = 0.2*stdev(nstart, nend, df1);
	
	swstart = nstart;
	swend = 400;
	swtest = 0; 
	
	/* Find first subwaveform */
	for (mwel=nstart; mwel<=400; mwel++) {
		if ((0.5*df2[mwel]) > ep1) {
			swstart = mwel;
		}
		else {
			continue;
		}
		
		for (mwel=swstart; mwel<=400; mwel++) {
			if (df1[mwel] > ep2) {
				continue;
			}
			else if ((mwel - swstart) > 3) {
				swend = mwel;
				swtest = 1;
				break;
			}
		}
	}
	
	swstart -= 8;
	swend += 8; 
	
	/* FILTERING SCHEME: Discard waveform if leading edge is not found */
	if (swtest == 0) {
        c.trackbits = 8192; 
	} 
	
	/* FILTERING SCHEME: Reject waveform if peak amplitude is not within first subwaveform 	  */
	
	if ( maxel > swend ) {
		c.trackbits = 8192; 
	} 
		
	// Calculate thermal noise level
	
	thn = 0.0;
	for (j=nstart; j<=nstart+nn; j++) {
		thn += wav[j]; 
	}
	thn = (double)thn/10.0;
 
	aval = 0.0; 	
 	anum = 0.0;
    aden = 0.0;
 	
 	for (j=nstart; j<nend; j++) {
        wp2 = ((double)wav[j]*(double)wav[j]);
        anum += (wp2*wp2); 
        aden += wp2;
    }
    
    aval=sqrt(anum/aden);
 	
 	/* FILTERING SCHEME:
		The basic idea is that the thermal noise / pre-arrival noise should not be more
		than around 20% of the amplitude value */
    
	if (thn > (0.2*aval)) {
		c.trackbits = 8192;
	}  	    
        
    /* Compute threshold level, according to Davis (1997), where A is the maximum waveform power */
        
    tlev = ((double)thn) + (q*(aval-(double)thn));
        
    /*-- 	Second loop to find threshold level
        		This works by finding minimum difference between element and tlpow --*/
     tlgate = 0.0;
           

    /*-- determine when the cpower exceeds the threshold and interpolate --*/

    k = nstart;
    while (wav[k] < tlev) {
    k++;
    
	if(k > nend) fprintf(stderr,"s20_thresh index out of bounds %d \n",k);
        }
		
	cgpow = wav[k];
    cgprev = wav[k-1];
        
    dpow=cgpow-cgprev;
        
    if(dpow > dpowmin){
        tlgate = (double)(k-1) + ((tlev-cgprev)/dpow);
    }
    else {
        tlgate = (double)(k-1) + 0.5; /* If the two consecutive gates have very similar values, place range value in between them */
    }       
        
    c.drange[2] = (int)(((double)tlgate-255.0)*gt2mm);
        
    	if ( (fwrite ((void *)&c, sizeof(struct CRYOSIN20HZ), (size_t)1, fpout)) != 1) {
			fprintf (stderr, "Error writing data to %s\n", argv[2]);
			return EXIT_FAILURE;
		}
		 	
	itest = c.trackbits;
	if (itest != 0 ) {
	 	nbad++;
	} 
	else {
		ngood++;
	} 	
	
	}
	
	fclose(fpin);
	fclose(fpout);
	fprintf(stdout,"%d %d %d \n",nw,ngood,nbad);
	return EXIT_SUCCESS;
}


double mean(int ns, int ne, double htd[]) {
    double	sum=0;
    int		i;
    
    for(i=ns; i<ne; i++)
        sum = sum + htd[i];
    return((double)sum/(ne-ns));
}

double stdev(int ns, int ne, double htd[]) {
    double	sum=0.0, mht=0.0;
    int		i;
    
    mht = mean(ns, ne, htd); 
    
    for(i=ns; i<ne; i++)
        sum = sum + ((htd[i]-mht)*(htd[i]-mht));
    return(sqrt((double)sum/(ne-ns-1)));

}