function[M0,Mt,Ms]=pcf_model(t,t0,sgt)
%
%       Create the error function model and its derivatives
%
%       input:
%
%       t       - times to evaluate model
%       t0      - arrival time
%       sgt     - sigma of model
%
%       output:
%
%       M0      - error function model
%       Mt      - derivative with respect to t0
%       Ms      - derivative with respect to sgt
%
%
%  compute eta
%

z = -(t-t0)/sgt; 
fac = 1.0/sqrt(2.0);
fac=1;
om = -0.5;
od = 0.5;
dv = 0;
dp = 0;
pcf = 0;
pcd = 0;
pdd = 0;

for i=1:length(t),
	[om,z(i),dv,dp,pcf,pdd] = pbdv(om,z(i),dv,dp,pcf,pdd);
	M0(i) = fac*pcf/sqrt(sgt);
	[od,z(i),dv,dp,pcd,pdd] = pbdv(od,z(i),dv,dp,pcd,pdd);
	Ms(i) = -(fac*sgt^(-1.5))*((0.5*pcf)-(z(i)*pcd));
	Mt(i) = -(fac*sgt^(-1.5))*pcd;
end;
