function [ PAR ] = PARAM_SD_SAR


%%%%%%%%%%%%%UNIVERSAL CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PAR.R_e=6378137;                        % Mean Earth Radius
PAR.f_e=1/298.257223563;                % Mean Earth Flattening
PAR.ecc_e=sqrt((2-PAR.f_e).*PAR.f_e);   % Mean Earth Eccentricty
PAR.c0=299792458;                       % Speed of light m/s
PAR.GM_e=398600.4418e9;                 % Earth gravitational constant m3s-2


%%%%%%%%%%%%%Geometrical Parameters  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PAR.Height=717e3;                       % Satellite COM Height over Elipsoid
PAR.Vs=7.5e3;                           % Satellite Orbital Velocity

%%%%%%%%%%%%% INSTRUMENT CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%ORBITAL CONSTANTS  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PAR.kappa=(1+PAR.Height./PAR.R_e);            % Earth sphericity Factor
PAR.Vb=PAR.Vs./PAR.kappa;                     % Beam Velocity

%%%%%%%%%%%%% INSTRUMENT CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PAR.f_0=13.575e9;                                          % Carrier frequency
PAR.lambda_0=PAR.c0./PAR.f_0;                              % Carrier wavelength
PAR.Bt=350e6;                                              % Transmitted Bandwidth 
PAR.Br=320e6;                                              % Received Bandwidth or Usable Bandwidth
PAR.PW_tx=51.0e-6;                                         % Tx Pulse Width
PAR.PW_rx=44.8e-6;                                         % Rx Pulse Width
PAR.Chirp_Slope=PAR.Br./PAR.PW_rx;                         %  Chirp Slope
PAR.DORIS_clock=10e6;                                      % Doris Clock
PAR.Sequencer_clock=8.*PAR.DORIS_clock;                    % Digital sequencer clock coming from 10 MHz platform clock
PAR.clock_count=1./PAR.Sequencer_clock;                    % Period of a clock count
PAR.PRI_SAR=4400.*PAR.clock_count;                         % PRI
%PAR.PRI_SAR=4500.*PAR.clock_count;                        % PRI
PAR.PRF_SAR=1./PAR.PRI_SAR;                                % PRF
PAR.N_burst=64;                                            % Number of pulses in a burst
PAR.Length_burst=PAR.N_burst.*PAR.PRI_SAR;                 % Length of a burst
PAR.Ls=(PAR.Length_burst).*PAR.Vs;                         % Synthetic Aperture Length

PAR.BRI=(934538+968).*PAR.clock_count;                     % BRI

PAR.Raz_max=PAR.lambda_0.*PAR.Height./(2.*PAR.Ls);         % Theorethical Azimuth Resolution

PAR.N_ml_max=PAR.N_burst.*PAR.Raz_max./(PAR.Vb.*PAR.BRI);  % Multilooked Looks Number

PAR.time_tag_bias=8e-3;

PAR.alpha_p=2.836e-9.*PAR.Br./(2.*sqrt(2.*log(2)));         % SAR Adimensional PTR Width


%%%%%%%%%%%%% ALGORITHM CONSTANTS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PAR.os_p=8;
PAR.os_q=2;
PAR.os=PAR.os_p./PAR.os_q;                                 % Oversampling Factor

%%%%%%%%%%%%% DATA CONSTANTS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PAR.dt=1./PAR.Br;                                          % Time Step in Data
PAR.Ns=128;                                                % Range Gates Number
PAR.N_center=64;
PAR.No=-10.*PAR.os;

%%%%%%%%ANTENNA CHARACTERIZATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PAR.G_0=10.^(42.6./10);                                    % Antenna Gain
PAR.dist_COM2ant=1.5855;                                   % Distance CoM and Antenna
PAR.Baseline_Length=1.1676;

PAR.theta_3x_real=deg2rad(1.06);                           %(degree) Antenna 3 dB beamwidth (along-track)
PAR.theta_3y_real=deg2rad(1.199200);                       %(degree) Antenna 3 dB beamwidth (cross-track)


PAR.alpha_x=8.*log(2)./(PAR.Height.^2.*PAR.theta_3x_real.^2);
PAR.alpha_y=8.*log(2)./(PAR.Height.^2.*PAR.theta_3y_real.^2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%% SCATTERING CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PAR.RMSS=repmat(inf.*pi./180,1,length(PAR.Height)); 
PAR.MSS=PAR.RMSS.^2;
PAR.nu=1./(1.*PAR.MSS);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%












