%
%  matlab script for three experiments to understand the correlation
%  between drange and sigma.
%
clf
clear
%
%  set the standard model t0=32, sig=2.0 (3.6-m wave height)
%
t0 = 32.;
t=8:56;
sig=2.;
A0=500;
%
%  compute the model and derivatives
%
figure(1)
[M0,Mt,Ms]=erf_model(t,t0,sig);
ex=exp(-0.022*(t-32.5));
subplot(2,1,1);plot(t,ex.*M0/5.,'k',t,Mt,'k--',t,Ms,'k:')
xlabel('gate number')
ylabel(' normalized power ');
title(' sig = 2.; 3.6-m swh ')
text(34,.13,'M/5')
text(22,.07,'dM/dsig')
text(25,-.1,'dM/dt')
%
%   now generate the appropriate weight function
%
W=(23+1.03*A0*M0)/sqrt(46);
subplot(2,1,2);plot(t,500*ex.*M0./W,'k',t,-A0*Mt./W,'k--',t,A0*Ms./W,'k:')
xlabel('gate number')
ylabel(' normalized power ');
title(' 1/weight = (23 + 1.03*M)/sqrt(46)  ')
text(21,4,'-dM/dsig')
text(33,2,'-dM/dt')
text(35,6.5,'dM/dA')
%
%  Experiment A. fit a waveform that has uniform noise using a model
%  with a uniform weight function
%
orient tall
figure(2)
Pn=(1.03*A0*M0)/sqrt(46);
W=ones(1,length(t));
for k=1:1000;
gn=randn(1,length(t));
noise=mean(Pn)*gn;
data=A0*M0+noise;
[X,newmod,misfit]=fit_model(data,A0,M0,Mt,Ms,W);
dt(k)=X(2);
ds(k)=X(3);
end
subplot(2,2,1);plot(t,data,'o',t,A0*M0,'k',t,newmod,'k--');
axis([20,56,-100,600])
xlabel(' gate number ')
ylabel(' power ')
title ('A, uniform error, uniform weight')
subplot(2,2,2);plot(ds,dt,'+')
axis([-1,1,-1,1])
grid
xlabel('delta sigma')
ylabel(' delta t')
title(' 1000 Gaussian Random Models ')
%
%  Experiment B , noise proportional to power, uniform weight
%
for k=1:1000;
gn=randn(1,length(t));
noise=Pn.*gn;
data=A0*M0+noise;
[X,newmod,misfit]=fit_model(data,A0,M0,Mt,Ms,W);
dt(k)=X(2);
ds(k)=X(3);
end
subplot(2,2,3);plot(t,data,'o',t,A0*M0,'k',t,newmod,'k--');
axis([20,56,-100,600])
xlabel(' gate number ')
ylabel(' power ')
title ('B, noise proportional to power, uniform weight')
subplot(2,2,4);plot(ds,dt,'+')
axis([-1,1,-1,1])
grid
xlabel('delta sigma')
ylabel(' delta t')
%
%  Experiment C , noise proportional to power, non-uniform weight
%
figure(3)
W=(23+1.03*A0*M0)/sqrt(46);
for k=1:1000;
gn=randn(1,length(t));
noise=W.*gn;
data=A0*M0+noise;
W=(23+1.03*data)/sqrt(46);
[X,newmod,misfit]=fit_model(data,A0,M0,Mt,Ms,W);
dt(k)=X(2);
ds(k)=X(3);
end
subplot(2,2,1);plot(t,data,'o',t,A0*M0,'k',t,newmod,'k--');
axis([20,56,-100,600])
xlabel(' gate number ')
ylabel(' power ')
title ('C, noise proportional to power, non-uniform weight')
subplot(2,2,2);plot(ds,dt,'+')
axis([-1,1,-1,1])
grid
xlabel('delta sigma')
ylabel(' delta t')
%
