function[X,newmod,misfit]=fit_model(data,A0,M0,Mt,Ms,W)
%
%       Create the error function model and its derivatives
%
%       input:
%
%       data    - power versus gate #
%       A0      - Actual known amplitude
%       M0      - error function, unit amplitude
%       Mt      - time derivative
%       Ms      - sigma derivative
%       W       - weight function for least squares
%
%       output:
%
%       X       - change in amp, time, sig
%       newmod  - new model
%       misfit  - chi squared
%
%
  Afit=sum(data)/sum(M0);
  B=(data-Afit*M0)./W; 
  A=[M0./W;A0*Mt./W;A0*Ms./W];   
  X=B/A;
  dmod=(A'*X')';   
  newmod=A0*M0+dmod;
  misfit=sqrt(sum(((data-newmod)./W).^2)/length(data));
  return
