%
%  matlab script for three experiments to understand the correlation
%  between drange and sigma.
%
clf
clear
%
%  set the standard model t0=64, sig=2.0 (3.6-m wave height)
%
t0 = 64.;
t=9:109;
sig=2.;
A0=65000;
%
%  compute the model and derivatives
%
figure(1)
[M0,Mt,Ms]=erf_model(t,t0,sig);
ex=exp(-0.013*(t-63.0));
subplot(2,1,1);plot(t,ex.*M0/5.,'k',t,Mt,'k--',t,Ms,'k:')
axis([40,112,-.3,.3])
xlabel('gate number')
ylabel(' normalized power ');
title(' sig = 2.; 3.6-m swh ')
text(68,.13,'M/5')
text(52,.07,'dM/dsig')
text(56,-.1,'dM/dt')
%
%   now generate the appropriate weight function
%
W=(5500+1.0*A0*M0)/9.8;
subplot(2,1,2);plot(t,A0*ex.*M0./W,'k',t,-A0*Mt./W,'k--',t,A0*Ms./W,'k:')
axis([40,112,-1.8,9.])
xlabel('gate number')
ylabel(' normalized power ');
title(' weighted ')
text(42,4,'-dM/dsig')
text(66,2,'-dM/dt')
text(70,6.5,'dM/dA')
%
%  Experiment A. fit a waveform that has uniform noise using a model
%  with a uniform weight function
%
orient tall
figure(2)
Pn=(1.0*A0*M0)/9.8;
W=ones(1,length(t));
for k=1:1000;
gn=randn(1,length(t));
noise=mean(Pn)*gn;
data=A0*M0+noise;
[X,newmod,misfit]=fit_model(data,A0,M0,Mt,Ms,W);
dt(k)=X(2);
ds(k)=X(3);
end
subplot(2,2,1);plot(t,data/A0,'o',t,M0,'k',t,newmod/A0,'k--');
axis([40,112,-.2,1.4])
xlabel(' gate number ')
ylabel(' power ')
title ('A, uniform error, uniform weight')
subplot(2,2,2);plot(ds,dt,'+')
axis([-1,1,-1,1])
grid
xlabel('delta sigma')
ylabel(' delta t')
title(' 1000 Gaussian Random Models ')
%
%  Experiment B , noise proportional to power, uniform weight
%
for k=1:1000;
gn=randn(1,length(t));
noise=Pn.*gn;
data=A0*M0+noise;
[X,newmod,misfit]=fit_model(data,A0,M0,Mt,Ms,W);
dt(k)=X(2);
ds(k)=X(3);
end
subplot(2,2,3);plot(t,data/A0,'o',t,M0,'k',t,newmod/A0,'k--');
axis([40,112,-.2,1.4])
xlabel(' gate number ')
ylabel(' power ')
title ('B, noise proportional to power, uniform weight')
subplot(2,2,4);plot(ds,dt,'+')
axis([-1,1,-1,1])
grid
xlabel('delta sigma')
ylabel(' delta t')
%
%  Experiment C , noise proportional to power, non-uniform weight
%
figure(3)
for k=1:1000;
gn=randn(1,length(t));
noise=Pn.*gn;
data=A0*M0+noise;
W=(5500+1.0*A0*M0)/9.8;
[X,newmod,misfit]=fit_model(data,A0,M0,Mt,Ms,W);
da(k)=X(1);
dt(k)=X(2);
ds(k)=X(3);
end
dt3=std(dt)
subplot(2,2,1);plot(t,data/A0,'o',t,M0,'k',t,newmod/A0,'k--');
axis([40,112,-.2,1.4])
xlabel(' gate number ')
ylabel(' power ')
title ('C, noise proportional to power, non-uniform weight')
subplot(2,2,2);plot(ds,dt,'+')
axis([-1,1,-1,1])
grid
xlabel('delta sigma')
ylabel(' delta t')
subplot(2,2,3);plot(dt,da/A0,'+')
xlim([-.5,.5])
xlabel('delta t')
ylabel(' delta A')
subplot(2,2,4);plot(ds,da/A0,'+')
xlim([-.5,.5])
xlabel('delta sigma')
ylabel(' delta A')
%
%
%  Experiment D , noise proportional to power, non-uniform weight
%
figure(4)
for k=1:1000;
gn=randn(1,length(t));
noise=Pn.*gn;
data=A0*M0+noise;
W=(5500+1.0*A0*M0)/9.8;
[X,newmod,misfit]=fit_model2(data,A0,M0,Mt,W);
da(k)=X(1);
dt(k)=X(2);
end
dt2=std(dt)
dt3/dt2
subplot(2,2,1);plot(t,data/A0,'o',t,M0,'k',t,newmod/A0,'k--');
axis([40,112,-.2,1.4])
xlabel(' gate number ')
ylabel(' power ')
title ('D,  2-par, noise proportional to power, non-uniform weight')
subplot(2,2,2);plot(dt,da/A0,'+')
xlim([-.5,.5])
xlabel('delta t')
ylabel(' delta A')
%
%
%  Experiment E , noise proportional to power, uniform weight
%
figure(5)
for k=1:1000;
gn=randn(1,length(t));
noise=Pn.*gn;
data=A0*M0+noise;
W=2500;
[X,newmod,misfit]=fit_model2(data,A0,M0,Mt,W);
da(k)=X(1);
dt(k)=X(2);
end
dt2=std(dt)
dt3/dt2
subplot(2,2,1);plot(t,data/A0,'o',t,M0,'k',t,newmod/A0,'k--');
axis([40,112,-.2,1.4])
xlabel(' gate number ')
ylabel(' power ')
title ('E,  2-par, noise proportional to power, uniform weight')
subplot(2,2,2);plot(dt,da/A0,'+')
xlim([-.5,.5])
xlabel('delta t')
ylabel(' delta A')
