/* iwetdry.c  Integer function iwetdry_(double *lat, double *lon) [Fortran callable]
	returns -1 on error, 0 over ocean areas, 1 over lakes, 2 over dry working from two
	2' pixel-registered global bitmask stored in WETDRY_MASKNAME and SEALND_MASKNAME.

WHF Smith 5 Aug 1995

*/

#include "iwetdry.h"

int iwetdry_(lat,lon)
double *lat, *lon;
{
	double	xlat, xlon;
	static int mask_needs_loading = 1;
	static unsigned char maskdata1[WETDRY_NXY8];
	static unsigned char maskdata2[WETDRY_NXY8];
	int	ilat, ilon, i, flag;
	FILE	*fp;

	if (mask_needs_loading) {
		if ( (fp = fopen(WETDRY_MASKNAME, "r")) == NULL) {
			fprintf(stderr,"wetdry:  ERROR.  Cannot open %s\n", WETDRY_MASKNAME);
			return(-1);
		}
		if ( (fread((char *)maskdata1, 1, WETDRY_NXY8, fp)) != WETDRY_NXY8) {
			fprintf(stderr,"wetdry:  ERROR.  Cannot read %s\n", WETDRY_MASKNAME);
			return(-1);
		}
		fclose(fp);
		if ( (fp = fopen(SEALND_MASKNAME, "r")) == NULL) {
			fprintf(stderr,"wetdry:  ERROR.  Cannot open %s\n", SEALND_MASKNAME);
			return(-1);
		}
		if ( (fread((char *)maskdata2, 1, WETDRY_NXY8, fp)) != WETDRY_NXY8) {
			fprintf(stderr,"wetdry:  ERROR.  Cannot read %s\n", WETDRY_MASKNAME);
			return(-1);
		}
		fclose(fp);
		mask_needs_loading=0;
	}

	xlat = 90.0 - (*lat);
	xlon = (*lon);
	while (xlon < 0.0) xlon += 360.0;
	while (xlon >= 360.0) xlon -= 360.0;
	if (xlat < 0.0 || xlat > 180.0) {
		fprintf(stderr,"wetdry:  ERROR.  Latitude out of range. %f \n",*lat);
		return(-1);
	}
	ilat = floor(xlat * WETDRY_D);
	if (ilat == WETDRY_NY) ilat--;	/* South pole moves back into last row  */
	ilon = floor(xlon * WETDRY_D);

	i = ilat * WETDRY_NX + ilon;
        flag=0;
	if (maskdata1[i/8] & maskset[i%8]) flag=1;
	if (maskdata2[i/8] & maskset[i%8]) flag=flag+1;
        return(flag);
}
	
