/*  s20_ocog.c input.c20 output.c20 baseline */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cryosat20hz.h"

int main (int argc, char **argv) {

	struct CRYOSIN20HZ c;
	FILE	*fpin, *fpout;
	int		nw, i, j, k, bline;
	int		nn=5; /* number of gates taken to estimate thermal noise */
	int		nstart=14, nend=497; /* for clipping waveform at both ends to avoid Fourier wrap around issues */
	double	anum, aden, aval, wp2, wval; /* numerator of A, denominator of A, value of A, waveform power squared, value of W */
	double	cog, cognum; /* center of gravity value, denominator of center of gravity */
	double	cgpow, norm, tlev, tlpow, tlgate, cgprev, dpow, dpowmin = 1/1.28e5;
	double  gt2mm;
	int		pnw[nn];
	double	thn; /* thermal noise level */
	double	wav[512];
	int		coridx;
	double	agccor, wpow, scpow, rpow;
	unsigned int itest;
	
	const int agccortab[64] = { \
        6148, 6062, 5959, 5866, 5774, 5675, 5560, 5468, \
        5376, 5277, 5186, 5075, 4967, 4868, 4777, 4663, \
        4571, 4472, 4381, 4302, 4210, 4111, 3998, 3907, \
        3815, 3716, 3625, 3514, 3395, 3303, 3188, 3097, \
        3005, 2906, 2815, 2703, 2612, 2509, 2420, 2296, \
        2204, 2101, 2011, 1939, 1848, 1743, 1656, 1542, \
        1451, 1346, 1258, 1144, 1062, 941, 833, 742, \
        652, 544, 456, 342, 247, 147, 59, 0};
	
	/* Make sure usage is right by opening files  */
		
	if (argc != 4 || (fpin = fopen(argv[1], "r")) == NULL || (fpout = fopen(argv[2], "w")) == NULL) {
		fprintf (stderr, "usage:  s20_ocog input.s20 output.s20 baseline \n");
        	fprintf (stderr, "        baseline (1)- Baseline A \n");
    		fprintf (stderr, "        baseline (2)- Baseline B \n");
		return EXIT_FAILURE;
	}
	
	bline = atoi(argv[3]);
	
	/* The following could use error handling: */
	if(bline != 1 && bline != 2) {
    	fprintf(stderr,"invalid baseline value \n");
        return EXIT_FAILURE;
    }
        
	
    switch (bline) {
    	case 1:
			gt2mm=468.257;    	
    		break;
    	case 2:
    		gt2mm=234.129;
    		break;
    }
	
		
	nw = 0;	
	while ( (fread ((void *)&c, sizeof(struct CRYOSIN20HZ), (size_t)1, fpin)) == 1 ) {
		nw++;
		
		
	if (c.pamp[0] == 0) 
		coridx = 63;
	else if (bline == 1) /* Baseline A */
		coridx = (int)((c.pamp[0] + c.pamp[1])/100);
	else
		coridx = 62 - (int)((c.pamp[0] - c.pamp[1])/100);
			 
			 
	agccor = 1.0e-2 * (double)agccortab[coridx];

	for (i=nstart; i<nend; i++) {
	 	scpow = (double)c.wave[i]*c.esf_A*pow(2.0,c.esf_B);
        rpow = (10*log10(scpow)) + (0.01*(double)c.pamp[0]) - agccor;
		wav[i]=pow(10,0.1*rpow);
	}	

	/*-- First loop to get value of A --*/
    anum = 0.0;
    aden = 0.0;
    cognum = 0.0;
        
    // Calculate thermal noise level
	
	thn = 0.0;
	for (i=nstart; i<=nstart+nn; i++) {
		thn += (double)wav[i]; 
	}
        
    for (j=nstart; j<nend; j++) {
	    wp2 = ((double)wav[j]*(double)wav[j]);
	    anum += (wp2*wp2); 
	    aden += wp2;
	    cognum += (j*wp2);
    }
        
        
    aval=sqrt(anum/aden);
        
	/*-- Ad-hoc filtering scheme to eliminate bad waveforms
		The basic idea is that the thermal noise / pre-arrival noise should not be more
		than around 20% of the amplitude value --*/
    
	if (thn > (0.2*aval)) {
		itest = c.trackbits;
    if( (itest & 8192) == 0) {
        c.trackbits = c.trackbits + 8192; 
    }
		continue;
	}	  
        
    wval=(aden*aden)/(anum);
    cog=cognum/aden;
        
    /* Compute leading edge position, according to Wingham (1986) */
        
    tlgate = cog-(0.5*wval);
    c.drange[2] = (int)(((double)tlgate-255.0)*gt2mm);
        
    if ( (fwrite ((void *)&c, sizeof(struct CRYOSIN20HZ), (size_t)1, fpout)) != 1) {
		fprintf (stderr, "Error writing data to %s\n", argv[2]);
		return EXIT_FAILURE;
		} 	
	}
	
	fclose(fpin);
	fclose(fpout);
	
	//	fprintf(stderr,"number of waveforms in file: %d \n",nw);
	
	return EXIT_SUCCESS;
}
