/*  c20_dump.c input.c20 > output

program to read a Cryosat 20 Hz
and dump whatever.

  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cryosat20hz.h"

int main (int argc, char **argv) {

	struct	CRYOSIN20HZ c;
	FILE	*fpin;
	int		gate, bline, coridx;
    int	ngood=0, nbad=0;
	double	lat, lon, rpow, cpow, agccor, scpow, wpow;
	
	const int agccortab[64] = { \
        6148, 6062, 5959, 5866, 5774, 5675, 5560, 5468, \
        5376, 5277, 5186, 5075, 4967, 4868, 4777, 4663, \
        4571, 4472, 4381, 4302, 4210, 4111, 3998, 3907, \
        3815, 3716, 3625, 3514, 3395, 3303, 3188, 3097, \
        3005, 2906, 2815, 2703, 2612, 2509, 2420, 2296, \
        2204, 2101, 2011, 1939, 1848, 1743, 1656, 1542, \
        1451, 1346, 1258, 1144, 1062, 941, 833, 742, \
        652, 544, 456, 342, 247, 147, 59, 0};
	
	const double log2 = log10(2.0);
	
	/* Make sure usage is right by opening files  */
	
	if (argc != 3 || (fpin = fopen(argv[1], "r")) == NULL) {
		fprintf (stderr, "usage:  c20_pow input.c20 baseline \n");
		fprintf (stderr, "baseline (1)- Baseline A \n");
        fprintf (stderr, "baseline (2)- Baseline B \n");
		exit (-1);
	}

	bline = atoi(argv[2]);

	/* The following could use error handling: */
    if(bline != 1 && bline != 2) {
    	fprintf(stderr,"invalid baseline value \n");
        return EXIT_FAILURE;
    }
    
	while ( (fread ((void *)&c, sizeof(struct CRYOSIN20HZ), (size_t)1, fpin)) == 1 ) {

		lat = 1.0e-06 * c.lat;
		lon = 1.0e-06 * c.lon;
		
		
		if (c.pamp[0] == 0) 
			coridx = 63;
		else if (bline == 1) /* Baseline A */
			coridx = (int)((c.pamp[0] + c.pamp[1])/100);
		else
			coridx = 62 - (int)((c.pamp[0] - c.pamp[1])/100);
			 
			 
		agccor = 1.0e-2 * (double)agccortab[coridx];
		

 
		/* dump raw waveforms */
        for (gate=0; gate<512; gate++){
        	/* echo_power in watts = waveform_counts*(A*10^-9)*2^B */
        	
        	
        	
        	scpow = (double)c.wave[gate]*c.esf_A*pow(2.0,c.esf_B);
        	rpow = (10*log10(scpow)) + (0.01*(double)c.pamp[0]) - agccor;
        	
        	wpow = pow(10,0.1*rpow)*1.0e15/(double)c.csum;
        	// wpow = pow(10,0.1*rpow);
        	
        	// cpow = (double)c.wave[gate]/(double)c.csum;
	    	
	    	
	    	fprintf(stdout, "%d %d %d %g %g \n",c.esf_A,c.esf_B,c.wave[gate],rpow,wpow,scpow);
	    	// fprintf(stdout,"%g %d \n",(double)c.esf_A*1.0e-09,c.esf_B);        	
	    	// fprintf(stdout, "%lf %lf %d %g %g \n",lon,lat,c.wave[gate],rpow,cpow);
        }
	}
	fclose (fpin);
	exit (0);
}
