/*  s20_tlh input.c20 > output

program to read a Cryosat 20 Hz
and dump whatever.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cryosat20hz.h"

int main (int argc, char **argv) {

	struct CRYOSIN20HZ c;
	FILE	*fpin;
	int	i, n, k, tide, delay, h, h0, h1, h2,issb, kk, ioff, boff, nstr1, nstr2, nstr3, tide_csr;
    int     ngood=0, nbad=0;
	unsigned int flag, itest;
	double	t,lat, lon, t0, alt, range, ssh, diff, wfac, sfac=0., swdiff, damp;
        double  rhtd, rhd1, rmean, rm2, rms;
        double  rht[20];
	
	/* Make sure usage is right by opening files  */
	
	if (argc != 5 || (fpin = fopen(argv[1], "r")) == NULL) {
		fprintf (stderr, "usage:  s20_tlh input.c20 nstruct baseline mss \n");
        	fprintf (stderr, "        nstruct (1) - DO NOT apply corrections and tides \n");
        	fprintf (stderr, "        nstruct (2) - apply corrections and tides WITHOUT EDITING \n");
		fprintf (stderr, "        nstruct (3) - apply corrections and tides WITH EDITING \n");
		fprintf (stderr, " \n");
		fprintf (stderr, "        baseline (1) - Baseline A \n");
		fprintf (stderr, "        baseline (2) - Baseline B \n");
		fprintf (stderr, " \n");
		fprintf (stderr, "        mss (1) - no mean sea surface heights \n");
                fprintf (stderr, "        mss (2) - print out mean sea surface heights in 4th column \n");
		exit (-1);
	}
    	nstr1 = atoi(argv[2]);
	nstr2 = atoi(argv[3]);
	nstr3 = atoi(argv[4]);
	k = 0;
	n = 0;

	// if (nstr2 == 2) fprintf(stderr,"Applying offset to Baseline B data. \n");
		
	while ( (fread ((void *)&c, sizeof(struct CRYOSIN20HZ), (size_t)1, fpin)) == 1 ) {
		if(k == 0) t0 = 1.0e-06 * c.microsec + c.sec2000;
        k=k+1;
	t = (1.0e-06 * c.microsec) + (double)c.sec2000; 
	lat = 1.0e-06 *((double)c.lat);
	lon = 1.0e-06 *((double)c.lon);
	tide = c.hltide + c.hstide + c.hptide + c.hotide;
    	tide_csr = c.new_tide + c.hstide;
	/* don't use ionosphere for envisat */
	delay = c.hwet + c.hdry + c.hinvb;
   	ioff = 0.0;
	boff = 0.0;
	// ioff = -3440.0; /* based on analysis of ocean passes */
    	// boff = -238630.0; /* to be determined from offset of Baseline A and B */
	/* issb = floor(0.060*c.pswh[1]+.5); don't use issb for land! */
	flag = 0;
	
	itest = c.trackbits;	
	if( (itest & 0) == 0) flag = 0;
    else flag = 1;
		
	switch ( nstr1 ) {
		case 1:
			h=c.alt-(c.range+ioff)-c.drange[2];
			break;
		case 2:
			h=c.alt-(c.range+c.hdopp+tide+delay+ioff)-c.drange[2];
			break;
		case 3:
			if (abs(c.hinvb) < 1 | abs(c.hdry) < 1 | abs(c.hwet) < 1 ) {
				flag = 1;	
			}
			h=c.alt-(c.range+c.hdopp+tide+delay+ioff)-c.drange[2];
			break;			
	}

	if ( nstr2 == 2 ) {
		h=h+boff;
	}
                
	if ( (flag & 1) == 0) {
		switch (nstr3 ) {
			case 1: 
				printf("%lf %lf %lf %d \n",lon,lat,t,h);
				break;
			case 2:
				printf("%lf %lf %lf %d %d \n",lon,lat,t,h,c.mss);
                                break;

		} 
	}
	}
	fclose (fpin);
	exit (0);
}
